package ij.gui;

import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.awt.geom.AffineTransform;
import java.awt.Shape;
import ij.measure.Calibration;
import ij.ImagePlus;
import java.awt.Polygon;
import java.awt.Color;
import ij.Prefs;
import java.awt.BasicStroke;
import ij.WindowManager;
import ij.IJ;

/**
 * <p>Title: </p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2011</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
public abstract class AttributeRoi extends AbstractRoi{
    protected int type;
    protected int xMax, yMax;
    protected ImagePlus imp;
    private int imageID;
    protected static Color ROIColor = Prefs.getColor(Prefs.ROICOLOR,Color.yellow);
    protected static Color defaultFillColor;
    protected Color  strokeColor;
    protected Color fillColor;
    protected BasicStroke stroke;
    protected boolean wideLine;
    private String name;
    int cornerDiameter;
    private int position;
    private int channel, slice, frame;
    protected boolean drawDoubleClick;
    protected boolean fillDoubleClick;

    public AttributeRoi() {
    }


    /** Returns the ImagePlus associated with this ROI, or null. */
    public ImagePlus getImage() {
            return imp;
    }

    /** Returns the ID of the image associated with this ROI. */
    public int getImageID() {
            return imp!=null?imp.getID():imageID;
    }

    public int getType() {
            return type;
    }

    /** Returns Feret's diameter, the greatest distance between
            any two points along the ROI boundary. */
    public double getFeretsDiameter() {
            double[] a = getFeretValues();
            return a!=null?a[0]:0.0;
    }

    /** Caculates "Feret" (maximum caliper width), "FeretAngle"
            and "MinFeret" (minimum caliper width), "FeretX" and "FeretY". */
    public double[] getFeretValues() {
            double min=Double.MAX_VALUE, diameter=0.0, angle=0.0, feretX=0.0, feretY=0.0;
            int p1=0, p2=0;
            double pw=1.0, ph=1.0;
            if (imp!=null) {
                    Calibration cal = imp.getCalibration();
                    pw = cal.pixelWidth;
                    ph = cal.pixelHeight;
            }
            Polygon poly = getConvexHull();
            if (poly==null) {
                    poly = getPolygon();
                    if (poly==null) return null;
            }
            double w2=pw*pw, h2=ph*ph;
            double dx, dy, d;
            for (int i=0; i<poly.npoints; i++) {
                    for (int j=i; j<poly.npoints; j++) {
                            dx = poly.xpoints[i] - poly.xpoints[j];
                            dy = poly.ypoints[i] - poly.ypoints[j];
                            d = Math.sqrt(dx*dx*w2 + dy*dy*h2);
                            if (d>diameter) {diameter=d; p1=i; p2=j;}
                    }
            }
            Rectangle r = getBounds();
            double cx = r.x + r.width/2.0;
            double cy = r.y + r.height/2.0;
            int n = poly.npoints;
            double[] x = new double[n];
            double[] y = new double[n];
            for (int i=0; i<n; i++) {
                    x[i] = (poly.xpoints[i]-cx)*pw;
                    y[i] = (poly.ypoints[i]-cy)*ph;
            }
            double xr, yr;
            for (double a=0; a<=90; a+=0.5) { // rotate calipers in 0.5 degree increments
                    double cos = Math.cos(a*Math.PI/180.0);
                    double sin = Math.sin(a*Math.PI/180.0);
                    double xmin=Double.MAX_VALUE, ymin=Double.MAX_VALUE;
                    double xmax=-Double.MAX_VALUE, ymax=-Double.MAX_VALUE;
                    for (int i=0; i<n; i++) {
                            xr = cos*x[i] - sin*y[i];
                            yr = sin*x[i] + cos*y[i];
                            if (xr<xmin) xmin = xr;
                            if (xr>xmax) xmax = xr;
                            if (yr<ymin) ymin = yr;
                            if (yr>ymax) ymax = yr;
                    }
                    double width = xmax - xmin;
                    double height = ymax - ymin;
                    double min2 = Math.min(width, height);
                    min = Math.min(min, min2);
            }
            double x1=poly.xpoints[p1], y1=poly.ypoints[p1];
            double x2=poly.xpoints[p2], y2=poly.ypoints[p2];
            if (x1>x2) {
                    double tx1=x1, ty1=y1;
                    x1=x2; y1=y2; x2=tx1; y2=ty1;
            }
            feretX = x1*pw;
            feretY = y1*ph;
            dx=x2-x1; dy=y1-y2;
            angle = (180.0/Math.PI)*Math.atan2(dy*ph, dx*pw);
            if (angle<0) angle = 180.0 + angle;
            //breadth = getFeretBreadth(poly, angle, x1, y1, x2, y2);
            double[] a = new double[5];
            a[0] = diameter;
            a[1] = angle;
            a[2] = min;
            a[3] = feretX;
            a[4] = feretY;
            return a;
    }

    /** Returns the angle in degrees between the specified line and a horizontal line. */
    public double getAngle(int x1, int y1, int x2, int y2) {
            double dx = x2-x1;
            double dy = y1-y2;
            if (imp!=null && !IJ.altKeyDown()) {
                    Calibration cal = imp.getCalibration();
                    dx *= cal.pixelWidth;
                    dy *= cal.pixelHeight;
            }
            return (180.0/Math.PI)*Math.atan2(dy, dx);
    }


    public Polygon getConvexHull() {
            return getPolygon();
    }

    double getFeretBreadth(Shape shape, double angle, double x1, double y1, double x2, double y2) {
            double cx = x1 + (x2-x1)/2;
            double cy = y1 + (y2-y1)/2;
            AffineTransform at = new AffineTransform();
            at.rotate(angle*Math.PI/180.0, cx, cy);
            Shape s = at.createTransformedShape(shape);
            Rectangle2D r = s.getBounds2D();
            return Math.min(r.getWidth(), r.getHeight());
            /*
            ShapeRoi roi2 = new ShapeRoi(s);
            Roi[] rois = roi2.getRois();
            if (rois!=null && rois.length>0) {
                    Polygon p = rois[0].getPolygon();
                    ImageProcessor ip = imp.getProcessor();
                    for (int i=0; i<p.npoints-1; i++)
                            ip.drawLine(p.xpoints[i], p.ypoints[i], p.xpoints[i+1], p.ypoints[i+1]);
                    imp.updateAndDraw();
            }
            */
    }
    /** Sets the default (global) color used for ROI outlines.
     * @see #getColor()
     * @see #setStrokeColor(Color)
     */
    public static void setColor(Color c) {
            ROIColor = c;
    }

    /** Returns the default (global) color used for drawing ROI outlines.
     * @see #setColor(Color)
     * @see #getStrokeColor()
     */
    public static Color getColor() {
            return ROIColor;
    }

    /** Sets the color used by this ROI to draw its outline. This color, if not null,
     * overrides the global color set by the static setColor() method.
     * @see #getStrokeColor()
     * @see #setStrokeWidth(int)
     * @see ij.ImagePlus#setOverlay(ij.gui.Overlay)
     */
    public void setStrokeColor(Color c) {
             strokeColor = c;
    }

    /** Returns the the color used to draw the ROI outline or null if the default color is being used.
     * @see #setStrokeColor(Color)
     */
    public Color getStrokeColor() {
            return  strokeColor;
    }

    /** Sets the color used to fill ROIs when they are in an overlay.
     * @see ij.ImagePlus#setOverlay(ij.gui.Overlay)
     */
    public void setFillColor(Color color) {
            fillColor = color;
    }

    /** Returns the color used to fill this ROI, or null if it is not filled.
     * @see #getStrokeColor()
     */
    public Color getFillColor() {
            return fillColor;
    }
    public static void setDefaultFillColor(Color color) {
            defaultFillColor = color;
    }

    public static Color getDefaultFillColor() {
            return defaultFillColor;
    }
    /** Sets the width of the line used to draw this ROI.
     * @see #setStrokeColor(Color)
     * @see ij.ImagePlus#setOverlay(ij.gui.Overlay)
     */
    public void setStrokeWidth(float width) {
            //this.stroke = new BasicStroke(width);
            if (width==0)
                    stroke = null;
            else if (wideLine)
                    this.stroke = new BasicStroke(width, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
            else
                    this.stroke = new BasicStroke(width);
            if (width>1f) fillColor = null;
    }

    /** This is a version of setStrokeWidth() that accepts a double argument. */
    public void setStrokeWidth(double width) {
            setStrokeWidth((float)width);
    }

    /** Returns the lineWidth. */
    public float getStrokeWidth() {
            return stroke!=null?stroke.getLineWidth():1;
    }

    /** Sets the Stroke used to draw this ROI. */
    public void setStroke(BasicStroke stroke) {
            this.stroke = stroke;
    }

    /** Returns the Stroke used to draw this ROI, or null if no Stroke is used. */
    public BasicStroke getStroke() {
            return stroke;
    }

    /** Copy the attributes (outline color, fill color, outline width)
            of  'roi2' to the this selection. */
    public void copyAttributes(Roi roi2) {
            this. strokeColor = roi2. strokeColor;
            this.fillColor = roi2.fillColor;
            this.stroke = roi2.stroke;
    }

    /** Returns the name of this ROI, or null. */
    public String getName() {
            return name;
    }

    /** Sets the name of this ROI. */
    public void setName(String name) {
            this.name = name;
    }

    /** Sets the rounded rectangle corner diameter (pixels). */
    public void setCornerDiameter(int cornerDiameter) {
            if (cornerDiameter<0) cornerDiameter = 0;
            this.cornerDiameter = cornerDiameter;
            ImagePlus imp = WindowManager.getCurrentImage();
            if (imp!=null && this==imp.getRoi())
                    imp.updateAndDraw();
    }

    /** Returns the rounded rectangle corner diameter (pixels). */
    public int getCornerDiameter() {
            return cornerDiameter;
    }

    /** Obsolete; replaced by setCornerDiameter(). */
    public void setRoundRectArcSize(int cornerDiameter) {
            setCornerDiameter(cornerDiameter);
    }

    /** Obsolete; replaced by getCornerDiameter(). */
    public int getRoundRectArcSize() {
            return cornerDiameter;
    }

    /** Sets the stack position (image number) of this ROI. In an overlay, this
    * ROI is only displayed when the stack is at the specified position.
    * Set to zero to have the ROI displayed on all images in the stack.
    * @see ij.gui.Overlay
    */
    public void setPosition(int n) {
            if (n<0) n=0;
            position = n;
            channel = slice = frame = 0;
    }

    /** Returns the stack position (image number) of this ROI, or
    *  zero if the ROI is not associated with a particular stack image.
    * @see ij.gui.Overlay
    */
    public int getPosition() {
            return position;
    }

    /** Sets the hyperstack position of this ROI. In an overlay, this
    * ROI is only displayed when the hyperstack is at the specified position.
    * @see ij.gui.Overlay
    */
    public void setPosition(int channel, int slice, int frame) {
            if (channel<0) channel=0;
            this.channel = channel;
            if (slice<0) slice=0;
            this.slice = slice;
            if (frame<0) frame=0;
            this.frame = frame;
            position = 0;
    }
    /** Returns the channel position of this ROI, or zero
    *  if this ROI is not associated with a particular channel.
    */
    public final int getCPosition() {
            return channel;
    }

    /** Returns the slice position of this ROI, or zero
    *  if this ROI is not associated with a particular slice.
    */
    public final int getZPosition() {
            return slice;
    }

    /** Returns the frame position of this ROI, or zero
    *  if this ROI is not associated with a particular frame.
    */
    public final int getTPosition() {
            return frame;
    }

}
