/*package ij.plugin;

import ij.IJ;
import ij.Macro;
import ij.gui.GenericDialog;
import ij.io.SaveDialog;
import ij.plugin.frame.Editor;
import ij.text.TextWindow;

// This class creates a new macro or the Java source for a new plugin.
public class NewPlugin implements PlugIn {

	public static final int MACRO=0, JAVASCRIPT=1, PLUGIN=2, PLUGIN_FILTER=3, PLUGIN_FRAME=4, TEXT_FILE=5, TABLE=6;
    private static int rows = 16;
    private static int columns = 60;
    private static int tableWidth = 350;
    private static int tableHeight = 250;
    private int type = MACRO;
    private String name = "Macro.txt";
    private boolean monospaced;
    private boolean menuBar = true;
	private Editor ed;

    public void run(String arg) {
    	type = -1;
    	if (arg.startsWith("text")||arg.equals("")) {
    		type = TEXT_FILE;
    		name = "Untitled.txt";
    	} else if (arg.equals("macro")) {
    		type = MACRO;
    		name = "Macro.txt";
    	} else if (arg.equals("javascript")) {
    		type = JAVASCRIPT;
    		name = "Script.js";
     	} else if (arg.equals("plugin")) {
    		type = PLUGIN;
    		name = "My_Plugin.java";
    	} else if (arg.equals("frame")) {
    		type = PLUGIN_FRAME;
    		name = "Plugin_Frame.java";
    	} else if (arg.equals("filter")) {
    		type = PLUGIN_FILTER;
    		name = "Filter_Plugin.java";
    	} else if (arg.equals("table")) {
			String options = Macro.getOptions();
			if  (IJ.isMacro() && options!=null && options.indexOf("[Text File]")!=-1) {
    			type = TEXT_FILE;
    			name = "Untitled.txt";
    			arg = "text+dialog";
    		} else {
    			type = TABLE;
    			name = "Table";
    		}
    	}
    	menuBar = true;
    	if (arg.equals("text+dialog") || type==TABLE) {
			if (!showDialog()) return;
		}
		if (type==-1)
    		createPlugin("Converted_Macro.java", PLUGIN, arg);
		else if (type==MACRO || type==TEXT_FILE || type==JAVASCRIPT) {
			if (type==TEXT_FILE && name.equals("Macro"))
				name = "Untitled.txt";
			createMacro(name);
		} else if (type==TABLE)
			createTable();
		else
			createPlugin(name, type, arg);
    }

	public void createMacro(String name) {
		int options = (monospaced?Editor.MONOSPACED:0)+(menuBar?Editor.MENU_BAR:0);
		ed = new Editor(rows, columns, 0, options);
		if (type==MACRO && !name.endsWith(".txt"))
			name = SaveDialog.setExtension(name, ".txt");
		else if (type==JAVASCRIPT && !name.endsWith(".js")) {
			if (name.equals("Macro")) name = "script";
			name = SaveDialog.setExtension(name, ".js");
		}
		ed.create(name, "");
	}

	void createTable() {
			new TextWindow(name, "", tableWidth, tableHeight);
	}

	public void createPlugin(String name, int type, String methods) {
  		ed = (Editor)IJ.runPlugIn("ij.plugin.frame.Editor", "");
		if (ed==null) return;
		String pluginName = name;
		if (!(name.endsWith(".java") || name.endsWith(".JAVA")))
			name = SaveDialog.setExtension(name, ".java");
		String className = pluginName.substring(0, pluginName.length()-5);
		String text = "";
		text += "import ij.*;\n";
		text += "import ij.process.*;\n";
		text += "import ij.gui.*;\n";
		text += "import java.awt.*;\n";
		switch (type) {
			case PLUGIN:
				text += "import ij.plugin.*;\n";
				text += "import ij.plugin.frame.*;\n";
				text += "\n";
				text += "public class "+className+" implements PlugIn {\n";
				text += "\n";
				text += "\tpublic void run(String arg) {\n";
				if (methods.equals("plugin"))
					text += "\t\tIJ.showMessage(\""+className+"\",\"Hello world!\");\n";
				else
					text += methods;
				text += "\t}\n";
				break;
			case PLUGIN_FILTER:
				text += "import ij.plugin.filter.*;\n";
				text += "\n";
				text += "public class "+className+" implements PlugInFilter {\n";
				text += "\tImagePlus imp;\n";
				text += "\n";
				text += "\tpublic int setup(String arg, ImagePlus imp) {\n";
				text += "\t\tthis.imp = imp;\n";
				text += "\t\treturn DOES_ALL;\n";
				text += "\t}\n";
				text += "\n";
				text += "\tpublic void run(ImageProcessor ip) {\n";
				text += "\t\tip.invert();\n";
				text += "\t\timp.updateAndDraw();\n";
				text += "\t\tIJ.wait(500);\n";
				text += "\t\tip.invert();\n";
				text += "\t\timp.updateAndDraw();\n";
				text += "\t}\n";
				break;
			case PLUGIN_FRAME:
				text += "import ij.plugin.frame.*;\n";
				text += "\n";
				text += "public class "+className+" extends PlugInFrame {\n";
				text += "\n";
				text += "\tpublic "+className+"() {\n";
				text += "\t\tsuper(\""+className+"\");\n";
				text += "\t\tJTextArea ta = new JTextArea(15, 50);\n";
				text += "\t\tadd(ta);\n";
				text += "\t\tpack();\n";
				text += "\t\tGUI.center(this);\n";
				text += "\t\tshow();\n";
				text += "\t}\n";
				break;
		}
		text += "\n";
		text += "}\n";
		ed.create(pluginName, text);
	}

	public boolean showDialog() {
		String title;
		String widthUnit, heightUnit;
		int width, height;
		if (type==TABLE) {
			title = "New Table";
			name = "Table";
			width = tableWidth;
			height = tableHeight;
			widthUnit = "pixels";
			heightUnit = "pixels";
		} else {
			title = "New Text Window";
			name = "Untitled";
			width = columns;
			height = rows;
			widthUnit = "characters";
			heightUnit = "lines";
		}
		GenericDialog gd = new GenericDialog(title);
		gd.addStringField("Name:", name, 16);
		gd.addMessage("");
		gd.addNumericField("Width:", width, 0, 3, widthUnit);
		gd.addNumericField("Height:", height, 0, 3, heightUnit);
		if (type!=TABLE) {
			gd.setInsets(5, 30, 0);
			gd.addCheckbox("Menu Bar", true);
			gd.setInsets(0, 30, 0);
			gd.addCheckbox("Monospaced Font", monospaced);
		}
		gd.showDialog();
		if (gd.wasCanceled())
			return false;
		name = gd.getNextString();
		width = (int)gd.getNextNumber();
		height = (int)gd.getNextNumber();
		if (width<1) width = 1;
		if (height<1) height = 1;
		if (type!=TABLE) {
			menuBar = gd.getNextBoolean();
			monospaced = gd.getNextBoolean();
			columns = width;
			rows = height;
			if (rows>100) rows = 100;
			if (columns>200) columns = 200;
		} else {
			tableWidth = width;
			tableHeight = height;
			if (tableWidth<128) tableWidth = 128;
			if (tableHeight<75) tableHeight = 75;
		}
		return true;
	}

	// Returns the Editor the newly created macro or plugin was opened in.
	public Editor getEditor() {
		return ed;
	}

}*/

     package ij.plugin;
     import java.awt.*;
     import java.awt.event.ActionEvent;
     import java.awt.event.ActionListener;
     import ij.*;
     import ij.gui.*;
     import ij.plugin.frame.Editor;
     import ij.text.TextWindow;
     import javax.swing.JCheckBox;
     import javax.swing.JPanel;
     import javax.swing.Icon;
     import javax.swing.ImageIcon;
     import java.net.URL;
     import java.util.Vector;
     import javax.swing.JComponent;

     // This class creates a new macro or the Java source for a new plugin.
     public class NewPlugin implements PlugIn, ActionListener{

         public static final int MACRO=0, JAVASCRIPT=1, PLUGIN=2, PLUGIN_FILTER=3, PLUGIN_FRAME=4, TEXT_FILE=5, TABLE=6;

         private static int type = MACRO;
         private static String name = "Macro";
         private static String[] syntax = {"Text File", "Batch File", "C File", "CPP File", "HTML File", "IDL File",
                                            "JavaScript File", "Java File","Make File", "Properties File", "Perl File", "PHP File", "Pyton File",
                                                  "SQL File", "ShellScript File", "TeX File", "XML File"};
         private static String[] fileType = {".txt", ".bat", ".c", ".cpp", ".html", ".idl",
                                                    ".js", ".java",".make", ".props", ".pl", ".php", ".py",
                                                          ".sql", ".sh", ".tex", ".xml"};
         private static String[] iconNames = {"doc1", "doc2", "doc3", "doc4", "doc5", "doc6", "doc7", "doc8",
                                                "doc9", "doc23", "doc24", "doc18", "doc13", "doc14", "doc15", "doc16", "doc17" };
         private static int rows = 16;
         private static int columns = 60;
         private static boolean monospaced;
         private boolean menuBar = true;
             private Editor ed;
             private static int tableWidth = 350;
             private static int tableHeight = 250;
             private JCheckBox[] checkBoxes = new JCheckBox[18];
             private boolean[] lutChecked= new boolean[18];
             private static int rowIndex=6;
             private Vector<JComponent> lutCheckBox;

         public void run(String arg) {
             type = -1;
             //System.out.println("arg="+arg);
             if (arg.startsWith("text")||arg.equals("")) {
                     type = TEXT_FILE;
                     name = "Untitled.txt";
                 } else if (arg.equals("plugin")) {
                         type = PLUGIN;
                         name = "My_Plugin.java";
                 } else if (arg.equals("frame")) {
                         type = PLUGIN_FRAME;
                         name = "Plugin_Frame.java";
                 } else if (arg.equals("filter")) {
                         type = PLUGIN_FILTER;
                         name = "Filter_Plugin.java";
             } else if (arg.equals("table")) {
             String options = Macro.getOptions();
                 type = TABLE;
                 name = "Table";
             }
             menuBar = true;
             if (arg.equals("plugin") || arg.equals("filter") || arg.equals("frame")) {
                 createPlugin(name, type, arg);
                     }
                     else if (type==TABLE){
                         if (!showSizeDialog()) return;
                         createTable();
                     }
                     else if(arg.equals("text")){
                         if (!showDialog()) return;
                         createPlugin(name, type, arg);
                     }
             IJ.register(NewPlugin.class);
         }

         void createTable() {
                         new TextWindow(name, "", tableWidth, tableHeight);
         }
             public void createMacro(String name) {
                 ed = (Editor)IJ.runPlugIn("ij.plugin.frame.Editor", "");
                 if (name.endsWith(".java"))
                         name = name.substring(0, name.length()-5);
                 if (type==MACRO && !(name.endsWith(".txt")))
                         name += ".txt";
                 ed.create(name, "");
             }

             void createTextWindow() {
                     String tableName = name;
                     if (tableName.equals("Macro")) tableName = "Table";
                     if (columns<128 || rows<75 )
                             new TextWindow(tableName, "", 350, 250);
                     else
                             new TextWindow(tableName, "", columns, rows);
             }

             public void createPlugin(String name, int type, String methods) {
                     ed = (Editor)IJ.runPlugIn("ij.plugin.frame.Editor", "");
                     if (ed==null) return;
                     String pluginName = name;
                     String className = "Untitled";
                     String text = "";
                     //System.out.println("pluginName="+pluginName);
                     if(type >= 2 && type <= 4 && pluginName.endsWith(".java")){
                         className = pluginName.substring(0,
                                 pluginName.length() - 5);
                         //System.out.println("className=" + pluginName);
                         switch (type) {
                         case PLUGIN:
                             text += "import ij.*;\n";
                             text += "import ij.process.*;\n";
                             text += "import ij.gui.*;\n";
                             text += "import java.awt.*;\n";
                             text += "import ij.plugin.*;\n";
                             text += "import ij.plugin.frame.*;\n";
                             text += "\n";
                             text += "public class " + className +
                                     " implements PlugIn {\n";
                             text += "\n";
                             text += "\tpublic void run(String arg) {\n";
                             if (methods.equals("plugin"))
                                 text += "\t\tIJ.showMessage(\"" + className +
                                         "\",\"Hello world!\");\n";
                             else
                                 text += methods;
                             text += "\t}\n";
                             text += "\n";
                             text += "}\n";
                             break;
                         case PLUGIN_FILTER:
                             text += "import ij.*;\n";
                             text += "import ij.process.*;\n";
                             text += "import ij.gui.*;\n";
                             text += "import java.awt.*;\n";
                             text += "import ij.plugin.filter.*;\n";
                             text += "\n";
                             text += "public class " + className +
                                     " implements PlugInFilter {\n";
                             text += "\tImagePlus imp;\n";
                             text += "\n";
                             text +=
                                     "\tpublic int setup(String arg, ImagePlus imp) {\n";
                             text += "\t\tthis.imp = imp;\n";
                             text += "\t\treturn DOES_ALL;\n";
                             text += "\t}\n";
                             text += "\n";
                             text += "\tpublic void run(ImageProcessor ip) {\n";
                             text += "\t\tip.invert();\n";
                             text += "\t\timp.updateAndDraw();\n";
                             text += "\t\tIJ.wait(500);\n";
                             text += "\t\tip.invert();\n";
                             text += "\t\timp.updateAndDraw();\n";
                             text += "\t}\n";
                             text += "\n";
                             text += "}\n";
                             break;
                         case PLUGIN_FRAME:
                             text += "import ij.*;\n";
                             text += "import ij.process.*;\n";
                             text += "import ij.gui.*;\n";
                             text += "import java.awt.*;\n";
                             text += "import javax.swing.*;\n";
                             text += "import ij.plugin.frame.*;\n";
                             text += "\n";
                             text += "public class " + className +
                                     " extends PlugInFrame {\n";
                             text += "\n";
                             text += "\tpublic " + className + "() {\n";
                             text += "\t\tsuper(\"" + className + "\");\n";
                             text +=
                                     "\t\tJTextArea ta = new JTextArea(15, 50);\n";
                             text += "\t\tgetContentPane().add(ta);\n";
                             text += "\t\tpack();\n";
                             text += "\t\tGUI.center(this);\n";
                             text += "\t\tsetVisible(true);\n";
                             text += "\t}\n";
                             text += "\n";
                             text += "}\n";
                             break;
                         }
                     }
                     ed.create(pluginName, text);
                     //ed.open("",name);
             }
             public boolean showSizeDialog() {
                     String title;
                     String widthUnit, heightUnit;
                     int width, height;
                     if (type==TABLE) {
                             title = "New Table";
                             name = "Table";
                             width = tableWidth;
                             height = tableHeight;
                             widthUnit = "pixels";
                             heightUnit = "pixels";
                     } else {
                             title = "New Text Window";
                             name = "Untitled";
                             width = columns;
                             height = rows;
                             widthUnit = "characters";
                             heightUnit = "lines";
                     }
                     GenericDialog gd = new GenericDialog(title);
                     gd.addStringField("Name:", name, 16);
                     gd.addMessage("");
                     gd.addNumericField("Width:", width, 0, 3, widthUnit);
                     gd.addNumericField("Height:", height, 0, 3, heightUnit);
                     if (type!=TABLE) {
                             gd.setInsets(5, 30, 0);
                             gd.addCheckbox("Menu Bar", true);
                             gd.setInsets(0, 30, 0);
                             gd.addCheckbox("Monospaced Font", monospaced);
                     }
                     gd.showDialog();
                     if (gd.wasCanceled())
                             return false;
                     name = gd.getNextString();
                     width = (int)gd.getNextNumber();
                     height = (int)gd.getNextNumber();
                     if (width<1) width = 1;
                     if (height<1) height = 1;
                     if (type!=TABLE) {
                             menuBar = gd.getNextBoolean();
                             monospaced = gd.getNextBoolean();
                             columns = width;
                             rows = height;
                             if (rows>100) rows = 100;
                             if (columns>200) columns = 200;
                     } else {
                             tableWidth = width;
                             tableHeight = height;
                             if (tableWidth<128) tableWidth = 128;
                             if (tableHeight<75) tableHeight = 75;
                     }
                     return true;
             }


             public boolean showDialog() {
                     GenericDialog gd = new GenericDialog("Choice Window");
                     JPanel choicePanel=new JPanel();
                     gd.addPanel(choicePanel, GridBagConstraints.CENTER, new Insets(0, 0, 0, 0));
                     gd.addMessage("");
                     gd.setInsets(5, 30, 0);
                     JPanel colorCheckBoxPanel=new JPanel();
                     colorCheckBoxPanel.add(gd.addCheckboxGroupPanel(rowIndex, checkBoxes.length/6, syntax, lutChecked));
                     gd.addPanel(colorCheckBoxPanel, GridBagConstraints.CENTER, new Insets(0, 0, 0, 0));
                     lutCheckBox=gd.getCheckboxes();
                     name = "Untitled" + fileType[type];
                     for(int i=0; i < syntax.length; i++){
                       checkBoxes[i]=(JCheckBox)lutCheckBox.elementAt(i);
                       checkBoxes[i].addActionListener(new ActionListener(){
                           public void actionPerformed(ActionEvent e){
                               JCheckBox box = (JCheckBox)e.getSource();
                               String synName = box.getText();
                               for(int i=0; i < syntax.length; i++){
                                   if(syntax[i].equals(synName)){
                                       type=i;
                                       name = "Untitled" + fileType[type];
                                       break;
                                   }
                               }

                           }
                       });
                         checkBoxes[i].setIcon(createIcon(iconNames[i]));
                     }
                     checkBoxes[0].setSelected(true);
                     gd.setInsets(0, 30, 0);
                     gd.showDialog();
                     if (gd.wasCanceled())
                             return false;
                     return true;
             }

             private Icon createIcon(String key){
                 Icon icon = null;
                 String dir = Prefs.getString("syntaxdir");
                 String fileName=key+".png";
                 String path = dir+fileName;
                 URL url = getClass().getResource(path);
                 if(url != null){
                     icon = new ImageIcon(url);
                 }
             return icon;
             }

             public void actionPerformed(ActionEvent e){
                 JCheckBox box = (JCheckBox)e.getSource();
                 String synName = box.getText();
                 for(int i=0; i < syntax.length; i++){
                     if(syntax[i].equals(synName)){
                         name = "Untitled" + fileType[i];
                         break;
                     }
                 }

             }
     /* Returns the Editor the newly created macro or plugin was opened i. */

             public Editor getEditor() {
                     return ed;
             }

     }


     /**/
