package ij2x.plugin;


/*
 * $Id: CollapsiblePane.java,v 1.1 o 2011/11/07 12:05:21 rawakorg $
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */


import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import java.util.Hashtable;
import java.util.LinkedHashMap;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
//import javax.swing.JComponent;
import javax.swing.ImageIcon;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
//import javax.swing.SwingConstants;

import javax.swing.border.Border;
import javax.swing.border.BevelBorder;

import ij.IJ;
import ij.ImageJ;
import ij.plugin.PlugIn;
import ij.Prefs;

import ij.WindowManager;
import ij.ImagePlus;

import ij2x.util.ResourceString;
import ij2x.util.BorderlessButton;
import ij2x.util.AWTImages;
import ij2x.util.ImageIcons;

import org.jdesktop.swingx.JXCollapsiblePane;
import org.jdesktop.swingx.JXCollapsiblePane.Direction;


    public class CollapsiblePane extends JPanel implements PlugIn,
      PropertyChangeListener,
          ActionListener,
              MouseListener,
                    MouseMotionListener,
                           Runnable{
    //private static final Logger LOG = Logger
    //        .getLogger(InteractiveTestCase.class.getName());
    //protected Point frameLocation = new Point(0,0);
    protected static ResourceBundle resources;
    protected ImagePlus imp;
    protected boolean systemLF;
    protected Icon icon;
    protected BevelBorder bb ;//new BevelBorder for LOWER or RAISED type
    protected Border grayBorder = BorderFactory.createBevelBorder(bb.RAISED, Color.gray, Color.lightGray);
    protected Border lightGrayBorder = BorderFactory.createBevelBorder(bb.LOWERED, Color.lightGray, Color.lightGray);
    protected JPanel[] commandPanel;
    protected JButton leftButton, rightButton, parentButton;
    public static JButton toolboxButton;

    private JXCollapsiblePane cp, control, lut, adjust, math, stack, icontrol, movie, overlay, trans, emptoy;

    protected BorderLayout layout;
    private String imageDir = "stIcon";
    private String[][] imgName;
    private String[] barName, barNames;
    private static String dir;
    private int increment = 9, oldInc=0, currentInc;
    private int length;
    protected ImageJ ij;
    private boolean running=false;
    private Thread timer;
    private JPanel collapsiblePane;
    private JLabel nextCollapsLabel, parentCollapsLabel;
    private int offset = 80;
    private int offsetHeight;
    private boolean direction;
    private JPopupMenu pm;
    private static Hashtable<Object, Action> barTable;
    private static Action latestAction;
    private int index = 0, indexstr=0;
    private static int tooltips = 0;

    public static Map<String,String> COMMAND_BARS;
    private static String[] tooltipAll;

    static {
        try {
            resources = ResourceBundle.getBundle("ij2x.plugin.frame.resources.CollapsiblePane",
                                                 Locale.getDefault());
            tooltipAll = ResourceString.tokenize(ResourceString.getResourceString(resources, "tooltip"));
            } catch (MissingResourceException mre) {
            System.err.println("resources/CollapsiblePane.properties not found");
            System.exit(1);
        }
      }


    public CollapsiblePane() {
        ij = IJ.getInstance();
        start();
    }

    public void run(String arg){
        ij = IJ.getInstance();
        createCollapsibleGUI();
    }


    public JPanel createCollapsibleGUI(){

        collapsiblePane = new JPanel();
        nextCollapsLabel=new JLabel("", JLabel.CENTER);
        parentCollapsLabel=new JLabel("", JLabel.CENTER);
        parentCollapsLabel.addMouseListener(this);
        collapsiblePane.addMouseMotionListener(this);
        barTable = new Hashtable<Object, Action>();
        Action[] actions = getActions();
        for (int i = 0; i < actions.length; i++) {
            Action a = actions[i];
            //System.out.println("Action.NAME("+i+")="+a.getValue(Action.NAME));
            barTable.put(a.getValue(Action.NAME), a);
        }
        pm=new JPopupMenu();

        dir = Prefs.getString(imageDir);
        Direction directionAll = Direction.LEFT;

        control = new JXCollapsiblePane(directionAll);
        lut = new JXCollapsiblePane(directionAll);
        stack = new JXCollapsiblePane(directionAll);
        math = new JXCollapsiblePane(directionAll);
        icontrol = new JXCollapsiblePane(directionAll);
        trans = new JXCollapsiblePane(directionAll);
        adjust = new JXCollapsiblePane(directionAll);
        movie = new JXCollapsiblePane(directionAll);
        overlay = new JXCollapsiblePane(directionAll);

        imgName = new String[5][3];
        imgName[0][0]= "play";
        imgName[0][1]= "play_ro";
        imgName[0][2]= "stop";
        imgName[1][0]= "play_left";
        imgName[1][1]= "play_left_ro";
        imgName[1][2]= "stop_left";

        imgName[2][0]= "stop";
        imgName[2][1]= "stop";
        imgName[2][2]= "stop";
        imgName[3][0]= "stop_left";
        imgName[3][1]= "stop_left";
        imgName[3][2]= "stop_left";

        imgName[4][0]= "s_toolbar";
        imgName[4][1]= "s_toolbar_ro";
        imgName[4][2]= "s_toolbar_press";

        //barName = new String[9];

        /*barNames[indexstr++]=COMMAND_BARS.get("lutbar");
        barNames[indexstr++]=COMMAND_BARS.get("stackbar");
        barNames[indexstr++]=COMMAND_BARS.get("mathbar");
        barNames[indexstr++]=COMMAND_BARS.get("imagetypebar");
        barNames[indexstr++]=COMMAND_BARS.get("transformbar");
        barNames[indexstr++]=COMMAND_BARS.get("adjustbar");
        barNames[indexstr++]=COMMAND_BARS.get("moviebar");
        barNames[indexstr++]=COMMAND_BARS.get("overlaybar");
        barNames[indexstr]=COMMAND_BARS.get("commandbar");*/

        //indexstr = 0;

        String[] barnameAll = ResourceString.tokenize(ResourceString.getResourceString(resources, "barname"));
        barName = new String[barnameAll.length];
        for(int i=0; i < barnameAll.length; i++){
                barName[i] = ResourceString.
                        getResourceString(resources, barnameAll[i]);
                //System.out.println(tooltips[i]);

        }

        /*barName[indexstr++]="LUT Bar";
        barName[indexstr++]="Stack Bar";
        barName[indexstr++]="Math Bar";
        barName[indexstr++]="Imagetype Bar";
        barName[indexstr++]="Transform Bar";
        barName[indexstr++]="Adjust Bar";
        barName[indexstr++]="Movie Bar";
        barName[indexstr++]="Overlay Bar";
        barName[indexstr]="Command Bar";*/

        indexstr=0;
        commandPanel=new JPanel[barnameAll.length];
        length = commandPanel.length;
        commandPanel[indexstr]=new ControlBarPanel();
        commandPanel[indexstr]=((ControlBarPanel)commandPanel[indexstr++]).createControlBar();
        commandPanel[indexstr]=new LutPanel();
        commandPanel[indexstr]=((LutPanel)commandPanel[indexstr++]).createControlBar();
        commandPanel[indexstr]=new StackControlPanel();
        commandPanel[indexstr]=((StackControlPanel)commandPanel[indexstr++]).controlButton();
        commandPanel[indexstr]=new MathPanel();
        commandPanel[indexstr]=((MathPanel)commandPanel[indexstr++]).createControlBar();
        commandPanel[indexstr]=new ImageTypePanel();
        commandPanel[indexstr]=((ImageTypePanel)commandPanel[indexstr++]).createControlBar();
        commandPanel[indexstr]=new TransformPanel();
        commandPanel[indexstr]=((TransformPanel)commandPanel[indexstr++]).controlButton();
        commandPanel[indexstr]=new AdjustPanel();
        commandPanel[indexstr]=((AdjustPanel)commandPanel[indexstr++]).createControlBar();
        commandPanel[indexstr]=new MovieControllerPanel();
        commandPanel[indexstr]=((MovieControllerPanel)commandPanel[indexstr++]).controlButton();
        commandPanel[indexstr]=new OverlayPanel();
        commandPanel[indexstr]=((OverlayPanel)commandPanel[indexstr]).createControlBar();
        indexstr=0;
        control.add(commandPanel[indexstr++]);
        lut.add(commandPanel[indexstr++]);
        stack.add(commandPanel[indexstr++]);
        math.add(commandPanel[indexstr++]);
        icontrol.add(commandPanel[indexstr++]);
        trans.add(commandPanel[indexstr++]);
        adjust.add(commandPanel[indexstr++]);
        movie.add(commandPanel[indexstr++]);
        overlay.add(commandPanel[indexstr]);

        ImageIcon b1o = ImageIcons.getIcons(this);

        AWTImages.setImageDir("stIcon");


    String[] iconAllName = ResourceString.tokenize(ResourceString.getResourceString(resources, "collapsbar"));
    String[] tooltip = new String[tooltipAll.length];
    for(int i=0; i < tooltipAll.length; i++){
            tooltip[i] = ResourceString.
                    getResourceString(resources, tooltipAll[i]);
            //System.out.println(tooltips[i]);

    }

    barNames = new String[tooltipAll.length];

    Map<String, String> toolMap = new LinkedHashMap<String, String>();
    for(int i=0; i<tooltipAll.length; i++){
        toolMap.put(tooltipAll[i], tooltip[i]);
    }
    /*toolMap.put(tooltipAll[tooltips], tooltip[tooltips++]
               );
    toolMap.put(tooltipAll[tooltips], tooltip[tooltips++]
               );
    toolMap.put(tooltipAll[tooltips], tooltip[tooltips++]
               );
    toolMap.put(tooltipAll[tooltips], tooltip[tooltips++]
               );
    toolMap.put(tooltipAll[tooltips], tooltip[tooltips++]
               );
    toolMap.put(tooltipAll[tooltips], tooltip[tooltips++]
               );
    toolMap.put(tooltipAll[tooltips], tooltip[tooltips++]
               );
    toolMap.put(tooltipAll[tooltips], tooltip[tooltips++]
               );
    toolMap.put(tooltipAll[tooltips], tooltip[tooltips]
               );*/
    COMMAND_BARS=toolMap;
    indexstr=0;
    tooltips=0;
    for(int i=0; i < tooltipAll.length; i++){
        barNames[i]=COMMAND_BARS.get(tooltipAll[i]);
        //System.out.println(barNames[i]);
    }
    /*barNames[indexstr++]=COMMAND_BARS.get(tooltipAll[tooltips++]);
    barNames[indexstr++]=COMMAND_BARS.get(tooltipAll[tooltips++]);
    barNames[indexstr++]=COMMAND_BARS.get(tooltipAll[tooltips++]);
    barNames[indexstr++]=COMMAND_BARS.get(tooltipAll[tooltips++]);
    barNames[indexstr++]=COMMAND_BARS.get(tooltipAll[tooltips++]);
    barNames[indexstr++]=COMMAND_BARS.get(tooltipAll[tooltips++]);
    barNames[indexstr++]=COMMAND_BARS.get(tooltipAll[tooltips++]);
    barNames[indexstr++]=COMMAND_BARS.get(tooltipAll[tooltips++]);
    barNames[indexstr]=COMMAND_BARS.get(tooltipAll[tooltips]);*/

    Image[][] icons = new Image[iconAllName.length][3];
    for(int i=0; i < iconAllName.length; i++){
        for(int j=0; j < 3; j++){
            icons[i][j]=((ImageIcon)AWTImages.createBufferedIcon(
                    this
                      , ResourceString.tokenize(ResourceString.getResourceString(resources, iconAllName[i]))[j]
                        , b1o.getImage()
                          , 24, 1)).getImage();
        }
    }

        leftButton=BorderlessButton.createButton(this, leftButton, "Next: LUT Bar", icons[index++]);
        rightButton=BorderlessButton.createButton(this, rightButton, "Next: Movie Bar", icons[index]);
        iconAllName = ResourceString.tokenize(ResourceString.getResourceString(resources, "toolboxbar"));
        for(int i=0; i < iconAllName.length; i++){
            for(int j=0; j < 3; j++){
                icons[i][j]=((ImageIcon)AWTImages.createBufferedIcon(
                        this
                          , ResourceString.tokenize(ResourceString.getResourceString(resources, iconAllName[i]))[j]
                            , b1o.getImage()
                              , 24, 0)).getImage();
            }
        }
        toolboxButton=BorderlessButton.createButton(this, toolboxButton, "Tool Box", icons[0]);

        this.addPropertyChangeListener(this);

        collapsiblePane.setLayout(new GridBagLayout());

        nextCollapsLabel.setText("ToolBox");
        nextCollapsLabel.setEnabled(false);
        offsetHeight=(int)leftButton.getPreferredSize().getHeight();
        if(IJ.isMacintosh()){offset=90;}
        parentCollapsLabel.setMinimumSize(new Dimension(offset, offsetHeight));
        parentCollapsLabel.setMaximumSize(new Dimension(offset, offsetHeight));
        parentCollapsLabel.setPreferredSize(new Dimension(offset, offsetHeight));
        parentCollapsLabel.setText("Command Bar");

        /** set collapsiblePane  for MAC OSX **/
        if(true/*IJ.isMacintosh()*/){
            parentCollapsLabel.setToolTipText(barNames[7]);
            collapsiblePane.add(leftButton,
                                new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
                    , GridBagConstraints.WEST, GridBagConstraints.NONE,
                    new Insets(0, 0, 0, 0), 0, 0));
            collapsiblePane.add(parentCollapsLabel,
                                new GridBagConstraints(1, 0, 1, 1, 0.5, 0.0
                    , GridBagConstraints.WEST, GridBagConstraints.NONE,
                    new Insets(0, 0, 0, 0), 0, 0));
            collapsiblePane = stackPanels();
            collapsiblePane.add(rightButton,
                                new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0
                    , GridBagConstraints.EAST, GridBagConstraints.NONE,
                    new Insets(0, 0, 0, 0), 0, 0));
            collapsiblePane.add(toolboxButton,
                                new GridBagConstraints(4, 0, 1, 1, 0.0, 0.0
                    , GridBagConstraints.EAST, GridBagConstraints.NONE,
                    new Insets(0, 0, 0, 0), 0, 0));
            setIncrementCollapsedPane();
        }else
        {
            /*parentCollapsLabel.setToolTipText("Command Bar");
            collapsiblePane.add(parentCollapsLabel,
                                new GridBagConstraints(0, 0, 1, 1, 0.5, 0.0
                    , GridBagConstraints.WEST, GridBagConstraints.NONE,
                    new Insets(0, 0, 0, 0), 0, 0));
            collapsiblePane.add(control,
                                new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
                    , GridBagConstraints.CENTER, GridBagConstraints.NONE,
                    new Insets(0, 0, 0, 0), 0, 0));
            collapsiblePane.add(toolboxButton,
                                new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
                    , GridBagConstraints.EAST, GridBagConstraints.NONE,
                    new Insets(0, 0, 0, 0), 0, 0));*/
        }
        return collapsiblePane;
    }

    JPanel stackPanels(){
        collapsiblePane.add(control, new GridBagConstraints(2, 0, 1, 1, 0.5, 0.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        collapsiblePane.add(lut, new GridBagConstraints(2, 0, 1, 1, 0.5, 0.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        collapsiblePane.add(stack, new GridBagConstraints(2, 0, 1, 1, 0.5, 0.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        collapsiblePane.add(math, new GridBagConstraints(2, 0, 1, 1, 0.5, 0.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        collapsiblePane.add(icontrol, new GridBagConstraints(2, 0, 1, 1, 0.5, 0.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        collapsiblePane.add(trans, new GridBagConstraints(2, 0, 1, 1, 0.5, 0.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        collapsiblePane.add(adjust, new GridBagConstraints(2, 0, 1, 1, 0.5, 0.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        collapsiblePane.add(movie, new GridBagConstraints(2, 0, 1, 1, 0.5, 0.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        collapsiblePane.add(overlay, new GridBagConstraints(2, 0, 1, 1, 0.5, 0.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        return collapsiblePane;
    }


    // Action listener method
    public void actionPerformed(ActionEvent e){
        imp = WindowManager.getCurrentImage();
        //System.out.println("icount="+WindowManager.getImageCount());
        if (imp==null){
            ImageTypePanel.updateState();
            //return;
        }
           parentButton=(JButton)e.getSource();

           if(parentButton.equals(toolboxButton)){
               ImageJ ij = IJ.getInstance();
               ij.showToolbar();
               return;
           }

       if(parentButton.equals(leftButton)){
           if(!direction){
               direction = true;
           }
           if (increment == length){
               increment = 1;
           }
           else{
               increment++; //
           }
        }else if(parentButton.equals(rightButton)){
            if(direction){
                direction = false;
            }
            if (increment == 1)
                increment = length;
            else
                increment--;
        }
        if (increment == oldInc){
            oldInc--;
        }
        this.firePropertyChange("incrementId", oldInc, increment);
        oldInc=increment;
   }

    // Actionevent methods Mousemotionlistener
    public void mouseDragged(MouseEvent e) {}

    public void mouseMoved(MouseEvent e) {
        Rectangle boundLeft = leftButton.getBounds();
        Rectangle boundRight = rightButton.getBounds();
        Point inside = e.getPoint();
        collapsiblePane.repaint();//
        ImageTypePanel.updateState();//
        if(boundLeft.contains(inside) || boundRight.contains(inside)){
            switch (increment) {
            case 1: {
                leftButton.setToolTipText("Next: Stack Bar");
                rightButton.setToolTipText("Next: Command Bar");
                break;
            }
            case 2: {
                leftButton.setToolTipText("Next: Math Bar");
                rightButton.setToolTipText("Next: LUT Bar");
                break;
            }
            case 3: {
                leftButton.setToolTipText("Next: Imagetype Bar");
                rightButton.setToolTipText("Next: Stack Bar");
                break;
            }
            case 4: {
                leftButton.setToolTipText("Next: Transform Bar");
                rightButton.setToolTipText("Next: Math Bar");
                break;
            }
            case 5: {
                leftButton.setToolTipText("Next: Adjust Bar");
                rightButton.setToolTipText("Next: ImageType Bar");
                break;
            }
            case 6: {
                leftButton.setToolTipText("Next: Movie Bar");
                rightButton.setToolTipText("Next: Transform Bar");
                break;
            }
            case 7: {
                leftButton.setToolTipText("Next: Overlay Bar");
                rightButton.setToolTipText("Next: Adjust Bar");
                break;
            }
            case 8: {
                leftButton.setToolTipText("Next: Command Bar");
                rightButton.setToolTipText("Next: Movie Bar");
                break;
            }
            case 9: {
                leftButton.setToolTipText("Next: LUT Bar");
                rightButton.setToolTipText("Next: Overlay Bar");
                break;
            }
            }
        }
    }
    // PropertyChangeEvent method propertyChange
    public void propertyChange(PropertyChangeEvent e){
        nextCollapsLabel.setEnabled(false);
            if(e.getPropertyName().equals("incrementId") && direction){
                setIncrementCollapsedPane();
            }
            else if(e.getPropertyName().equals("incrementId") && !direction){
                setDecrementCollapsedPane();
            }
    }

    void setIncrementCollapsedPane(){
             switch (increment) {
             case 1: {
                 control.setCollapsed(!control.isCollapsed());
                 setVisible1(1);
                 parentCollapsLabel.setText(barName[increment-1]);
                 parentCollapsLabel.setToolTipText(barNames[increment-1]);
                 break;
             }
             case 2: {
                 lut.setCollapsed(!lut.isCollapsed());
                 setVisible2(2);
                 parentCollapsLabel.setText(barName[increment-1]);
                 parentCollapsLabel.setToolTipText(barNames[increment-1]);
                 break;
             }
             case 3: {
                 stack.setCollapsed(!stack.isCollapsed());
                 setVisible3(3);
                 parentCollapsLabel.setText(barName[increment-1]);
                 parentCollapsLabel.setToolTipText(barNames[increment-1]);
                 break;
             }
             case 4: {
                 math.setCollapsed(!math.isCollapsed());
                 setVisible4(4);
                 parentCollapsLabel.setText(barName[increment-1]);
                 parentCollapsLabel.setToolTipText(barNames[increment-1]);
                 break;
             }
             case 5: {
                 icontrol.setCollapsed(!icontrol.isCollapsed());
                 setVisible5(5);
                 parentCollapsLabel.setText(barName[increment-1]);
                 parentCollapsLabel.setToolTipText(barNames[increment-1]);
                 break;
             }
             case 6: {
                 trans.setCollapsed(!trans.isCollapsed());
                 setVisible6(6);
                 parentCollapsLabel.setText(barName[increment-1]);
                 parentCollapsLabel.setToolTipText(barNames[increment-1]);
                 break;
             }
             case 7: {
                 adjust.setCollapsed(!adjust.isCollapsed());
                 setVisible7(7);
                 parentCollapsLabel.setText(barName[increment-1]);
                 parentCollapsLabel.setToolTipText(barNames[increment-1]);
                 break;
             }
             case 8: {
                 movie.setCollapsed(!movie.isCollapsed());
                 setVisible8(8);
                 parentCollapsLabel.setText(barName[increment-1]);
                 parentCollapsLabel.setToolTipText(barNames[increment-1]);
                 break;
             }
             case 9: {
                 resetCollapsiblePane();
                 setVisible9(9);
                 parentCollapsLabel.setText(barName[increment-1]);
                 parentCollapsLabel.setToolTipText(barNames[increment-1]);
                 break;
             }
             }
         //}
}
     void setDecrementCollapsedPane(){
              switch (increment) {
              case 9: {
                  control.setCollapsed(!control.isCollapsed());
                  setVisible1(1);
                  parentCollapsLabel.setText(barName[increment-1]);
                  parentCollapsLabel.setToolTipText(barNames[increment-1]);
                  break;
              }
              case 1: {
                  lut.setCollapsed(!lut.isCollapsed());
                  setVisible2(2);
                  parentCollapsLabel.setText(barName[increment-1]);
                  parentCollapsLabel.setToolTipText(barNames[increment-1]);
                  break;
              }
              case 2: {
                  stack.setCollapsed(!stack.isCollapsed());
                  setVisible3(3);
                  parentCollapsLabel.setText(barName[increment-1]);
                  parentCollapsLabel.setToolTipText(barNames[increment-1]);
                  break;
              }
              case 3: {
                  math.setCollapsed(!math.isCollapsed());
                  setVisible4(4);
                  parentCollapsLabel.setText(barName[increment-1]);
                  parentCollapsLabel.setToolTipText(barNames[increment-1]);
                  break;
              }
              case 4: {
                  icontrol.setCollapsed(!icontrol.isCollapsed());
                  setVisible5(5);
                  parentCollapsLabel.setText(barName[increment-1]);
                  parentCollapsLabel.setToolTipText(barNames[increment-1]);
                  break;
              }
              case 5: {
                  trans.setCollapsed(!trans.isCollapsed());
                  setVisible6(6);
                  parentCollapsLabel.setText(barName[increment-1]);
                  parentCollapsLabel.setToolTipText(barNames[increment-1]);
                  break;
              }
              case 6: {
                  adjust.setCollapsed(!adjust.isCollapsed());
                  setVisible7(7);
                  parentCollapsLabel.setText(barName[increment-1]);
                  parentCollapsLabel.setToolTipText(barNames[increment-1]);
                  break;
              }
              case 7: {
                  movie.setCollapsed(!movie.isCollapsed());
                  setVisible8(8);
                  parentCollapsLabel.setText(barName[increment-1]);
                  parentCollapsLabel.setToolTipText(barNames[increment-1]);
                  break;
              }
              case 8: {
                  setCollapsiblePane();
                  setVisible9(9);
                  overlay.setCollapsed(!overlay.isCollapsed());
                  parentCollapsLabel.setText(barName[increment-1]);
                  parentCollapsLabel.setToolTipText(barNames[increment-1]);
                  break;
              }
              }
          }

          void setVisible1(int inc){
              if(1==inc){
                  control.setVisible(true);
                  lut.setVisible(direction);
                  stack.setVisible(false);
                  math.setVisible(false);
                  icontrol.setVisible(false);
                  trans.setVisible(false);
                  adjust.setVisible(false);
                  movie.setVisible(false);
                  overlay.setVisible(false);
              }
          }
          void setVisible2(int inc){
              if(2==inc){
                  control.setVisible(false);
                  lut.setVisible(true);
                  stack.setVisible(direction);
                  math.setVisible(false);
                  icontrol.setVisible(false);
                  trans.setVisible(false);
                  adjust.setVisible(false);
                  movie.setVisible(false);
                  overlay.setVisible(false);
              }
          }
          void setVisible3(int inc){
              if(3==inc){
                  control.setVisible(false);
                  lut.setVisible(false);
                  stack.setVisible(true);
                  math.setVisible(direction);
                  icontrol.setVisible(false);
                  trans.setVisible(false);
                  adjust.setVisible(false);
                  movie.setVisible(false);
                  overlay.setVisible(false);
              }
          }
          void setVisible4(int inc){
              if(4==inc){
                  control.setVisible(false);
                  lut.setVisible(false);
                  stack.setVisible(false);
                  math.setVisible(true);
                  icontrol.setVisible(direction);
                  trans.setVisible(false);
                  adjust.setVisible(false);
                  movie.setVisible(false);
                  overlay.setVisible(false);
              }
          }
          void setVisible5(int inc){
              if(5==inc){
                  control.setVisible(false);
                  lut.setVisible(false);
                  stack.setVisible(false);
                  math.setVisible(false);
                  icontrol.setVisible(true);
                  trans.setVisible(direction);
                  adjust.setVisible(false);
                  movie.setVisible(false);
                  overlay.setVisible(false);
              }
          }
          void setVisible6(int inc){
              if(6==inc){
                  control.setVisible(false);
                  lut.setVisible(false);
                  stack.setVisible(false);
                  math.setVisible(false);
                  icontrol.setVisible(false);
                  trans.setVisible(true);
                  adjust.setVisible(direction);
                  movie.setVisible(false);
                  overlay.setVisible(false);
              }
          }
          void setVisible7(int inc){
              if(7==inc){
                  control.setVisible(false);
                  lut.setVisible(false);
                  stack.setVisible(false);
                  math.setVisible(false);
                  icontrol.setVisible(false);
                  trans.setVisible(false);
                  adjust.setVisible(true);
                  movie.setVisible(direction);
                  overlay.setVisible(false);
              }
          }
          void setVisible8(int inc){
              if(8==inc){
                  control.setVisible(true);
                  lut.setVisible(false);
                  stack.setVisible(false);
                  math.setVisible(false);
                  icontrol.setVisible(false);
                  trans.setVisible(false);
                  adjust.setVisible(false);
                  movie.setVisible(true);
                  overlay.setVisible(direction);
              }
          }
          void setVisible9(int inc){
              if(9==inc){
                  control.setVisible(true);
                  lut.setVisible(false);
                  stack.setVisible(false);
                  math.setVisible(false);
                  icontrol.setVisible(false);
                  trans.setVisible(false);
                  adjust.setVisible(false);
                  movie.setVisible(false);
                  overlay.setVisible(true);
              }
          }
    void setCollapsiblePane(){

            for (int i = 1; i < length+1 ; i++) {
                switch (i) {
            case 1: {
                control.setCollapsed(!control.isCollapsed());
                break;
            }
            case 2: {
                lut.setCollapsed(!lut.isCollapsed());
                break;
            }
            case 3: {
                stack.setCollapsed(!stack.isCollapsed());
                break;
            }
            case 4: {
                math.setCollapsed(!math.isCollapsed());
                break;
            }
            case 5: {
                icontrol.setCollapsed(!icontrol.isCollapsed());
                break;
            }
            case 6: {
                trans.setCollapsed(!trans.isCollapsed());
                break;
            }
            case 7: {
                adjust.setCollapsed(!adjust.isCollapsed());
                break;
            }
            case 8: {
                movie.setCollapsed(!movie.isCollapsed());
                break;
            }
            case 9: {
                overlay.setCollapsed(!overlay.isCollapsed());
                break;
            }
            }
        }
    }
    void resetCollapsiblePane(){
        for (int i = 1; i < length+1 ; i++) {
            switch (i) {
            case 1: {
                if(i != increment && control.isCollapsed()){
                    control.setCollapsed(!control.isCollapsed());
                }
                break;
            }
            case 2: {
                if(i != increment && lut.isCollapsed()){
                    lut.setCollapsed(!lut.isCollapsed());
                }
                break;
            }
            case 3: {
                if(i != increment && stack.isCollapsed()){
                    stack.setCollapsed(!stack.isCollapsed());
                }
                break;
            }
            case 4: {
                if(i != increment && math.isCollapsed()){
                    math.setCollapsed(!math.isCollapsed());
                }
                break;
            }
            case 5: {
                if(i != increment && icontrol.isCollapsed()){
                    icontrol.setCollapsed(!icontrol.isCollapsed());
                }
                break;
            }
            case 6: {
                if(i != increment && trans.isCollapsed()){
                    trans.setCollapsed(!trans.isCollapsed());
                }
                break;
            }
            case 7: {
                if(i != increment && adjust.isCollapsed()){
                    adjust.setCollapsed(!adjust.isCollapsed());
                }
                break;
            }
            case 8: {
                if(i != increment && movie.isCollapsed()){
                    movie.setCollapsed(!movie.isCollapsed());
                }
                break;
            }
            case 9: {
                if(i != increment && overlay.isCollapsed()){
                    overlay.setCollapsed(!overlay.isCollapsed());
                }
                break;
            }
            }
        }
    }
    void addPopupMenu(JPopupMenu pm) {
             addPopupItem("command", pm);
             addPopupItem("lut", pm);
             addPopupItem("stack", pm);
             addPopupItem("math", pm);
             addPopupItem("imagetype", pm);
             addPopupItem("trans", pm);
             addPopupItem("adjust", pm);
             addPopupItem("movie", pm);
     }

     void addPopupItem(String cmd, JPopupMenu pm) {
             String astr = getResourceString(cmd + actionSuffix);
             String lstr = getResourceString(cmd + labelSuffix);
             JMenuItem mi= new JMenuItem(lstr);
             if (astr == null) {
                 astr = cmd;
             }
             mi.setActionCommand(astr);
             Action a = getAction(cmd);
             if (a != null) {
                 mi.addActionListener(a);
                 a.addPropertyChangeListener(this);
                 //a.addPropertyChangeListener(createActionChangeListener(mi));
                 mi.setEnabled(a.isEnabled());
             } else {
                 mi.setEnabled(false);
             }
             /*mi.addActionListener(new ActionListener(){
             public void actionPerformed(ActionEvent e){
                 action_Performed(e, mi);
             }
             });*/
             pm.add(mi);
     }
     // Yarked from JMenu, ideally this would be public.
     protected PropertyChangeListener createActionChangeListener(AbstractButton b) {
         return new ActionChangedListener(b);
     }

     // Yarked from JMenu, ideally this would be public.
     private class ActionChangedListener implements PropertyChangeListener {
         AbstractButton menuItem;

         ActionChangedListener(AbstractButton mi) {
             super();
             this.menuItem = mi;
         }
         public void propertyChange(PropertyChangeEvent e) {
             String propertyName = e.getPropertyName();
             //System.out.println("propertyName=" +propertyName);
            if (e.getPropertyName().equals(Action.NAME)) {
                 String text = (String) e.getNewValue();
                 menuItem.setText(text);
             } else if (propertyName.equals("enabled")) {
                 Boolean enabledState = (Boolean) e.getNewValue();
                 menuItem.setEnabled(enabledState.booleanValue());
             }
         }
     }


     /**
     * Fetch the list of actions supported by this
     * editor.  It is implemented to return the list
     * of actions supported by the embedded JTextComponent
     * augmented with the actions defined locally.
     */
    public Action[] getActions() {
        return allBarActions;
    }
     protected Action getAction(String cmd) {
         return barTable.get(cmd);
     }
     protected String getResourceString(String nm) {
         String str;
         try {
             str = resources.getString(nm);
         } catch (MissingResourceException mre) {
             str = null;
         }
         return str;
     }
     public void mousePressed (MouseEvent e) {
             int x=e.getX(), y=e.getY();
             if (e.isPopupTrigger() || e.isMetaDown())
                     pm.show(e.getComponent(),x,y);
     }
     public void mouseReleased (MouseEvent e) {}
     public void mouseClicked (MouseEvent e) {}
     public void mouseEntered (MouseEvent e) {}
     public void mouseExited (MouseEvent e) {}

     /**
      * Suffix applied to the key used in resource file
      * lookups for an image.
      */
     public static final String imageSuffix = "Image";

     /**
      * Suffix applied to the key used in resource file
      * lookups for a label.
      */
     public static final String labelSuffix = "Label";

     /**
      * Suffix applied to the key used in resource file
      * lookups for an action.
      */
     public static final String actionSuffix = "Action";

     public static final String commandAction = "command";
     public static final String lutAction = "lut";
     public static final String stackAction = "stack";
     public static final String mathAction = "math";
     public static final String imagetypeAction = "imagetype";
     public static final String transAction = "trans";
     public static final String adjustAction = "adjust";
     public static final String movieAction = "movie";

     private boolean isCommandAction = !true;
     private boolean isLutAction = !true;
     private boolean isStackAction = !true;
     private boolean isMathAction = !true;
     private boolean isImagetypeAction = !true;
     private boolean isTransAction = !true;
     private boolean isAdjustAction = !true;
     private boolean isMovieAction = !true;

     private Action[] allBarActions = {
                                       new CommandAction(),
                                       new LutAction(),
                                       new StackAction(),
                                       new MathAction(),
                                       new ImagetypeAction(),
                                       new TransAction(),
                                       new AdjustAction(),
                                       new MovieAction()
     };

     void setEnabledMenuItem(int increment){
         resetEnabledMenuItem();
         switch (increment) {
             case 1: {
                 isStackAction = false;
                 setEnabled(isStackAction);
                 break;
             }
             case 2: {
                 isMathAction = false;
                 setEnabled(isMathAction);
                 break;
             }
             case 3: {
                 isImagetypeAction = false;
                 setEnabled(isImagetypeAction);
                 break;
             }
             case 4: {
                 isTransAction = false;
                 setEnabled(isTransAction);
                 break;
             }
             case 5: {
                 isAdjustAction = false;
                 setEnabled(isAdjustAction);
                 break;
             }
             case 6: {
                 isMovieAction = false;
                 setEnabled(isMovieAction);
                 break;
             }
             case 7: {
                 isLutAction = false;
                 setEnabled(isLutAction);
                 break;
             }
             case 8: {
                 isCommandAction = false;
                 setEnabled(isCommandAction);
                 break;
             }

         }
     }
     void resetEnabledMenuItem(){
                 isStackAction = true;
                 isMathAction = true;
                 isImagetypeAction = true;
                 isTransAction = true;
                 isAdjustAction = true;
                 isMovieAction = true;
                 isLutAction = true;
                 isCommandAction = true;
     }

     class CommandAction extends AbstractAction {

         CommandAction() {
             super(commandAction);
             setEnabled(isCommandAction);
         }

         public void actionPerformed(ActionEvent e) {
             currentInc=increment;
             increment=8;
             this.firePropertyChange("incrementId", oldInc, increment);
             oldInc=increment;
         }
     }

     class LutAction extends AbstractAction {

         LutAction() {
             super(lutAction);
             setEnabled(isLutAction);
         }

         public void actionPerformed(ActionEvent e) {
             currentInc=increment;
             increment=1;
             this.firePropertyChange("incrementId", oldInc, increment);
             oldInc=increment;
         }
     }

     class StackAction extends AbstractAction {

         StackAction() {
             super(stackAction);
             setEnabled(isStackAction);
         }

         public void actionPerformed(ActionEvent e) {
             currentInc=increment;
             increment=2;
             this.firePropertyChange("incrementId", oldInc, increment);
             oldInc=increment;
         }
     }
     class MathAction extends AbstractAction {

         MathAction() {
             super(mathAction);
             setEnabled(isMathAction);
         }

         public void actionPerformed(ActionEvent e) {
             currentInc=increment;
             increment=3;
             this.firePropertyChange("incrementId", oldInc, increment);
             oldInc=increment;
         }
     }

     class ImagetypeAction extends AbstractAction {

         ImagetypeAction() {
             super(imagetypeAction);
             setEnabled(isImagetypeAction);
         }

         public void actionPerformed(ActionEvent e) {
             currentInc=increment;
             increment=4;
             this.firePropertyChange("incrementId", oldInc, increment);
             oldInc=increment;
         }
     }

     class TransAction extends AbstractAction {

         TransAction() {
             super(transAction);
             setEnabled(isTransAction);
         }

         public void actionPerformed(ActionEvent e) {
             currentInc=increment;
             increment=5;
             this.firePropertyChange("incrementId", oldInc, increment);
             oldInc=increment;
         }
     }
         class AdjustAction extends AbstractAction {

             AdjustAction() {
                 super(adjustAction);
                 setEnabled(isAdjustAction);
             }

             public void actionPerformed(ActionEvent e) {
                 currentInc=increment;
                 increment=6;
                 this.firePropertyChange("incrementId", oldInc, increment);
                 oldInc=increment;
            }
         }

         class MovieAction extends AbstractAction {

             MovieAction() {
                 super(movieAction);
                 setEnabled(isMovieAction);
             }

             public void actionPerformed(ActionEvent e) {
                 currentInc=increment;
                 increment=7;
                 this.firePropertyChange("incrementId", oldInc, increment);
                 oldInc=increment;
             }
     }

    /* interface Runnable methods */
    public void start() {
        running = true;
        if (timer == null)
            timer = new Thread(this);
        timer.start();
    }

    public void stop() {
        timer = null;
        notify();
    }

    public void run() {
        while (running) {
            try {
                /*if(direction){
                    increment++;
                    if(increment > length)increment=1;
                    switch (increment) {
                    case 1: {
                        rightButton.setToolTipText("Next: Stack Bar");
                        break;
                    }
                    case 2: {
                        rightButton.setToolTipText("Next: Math Bar");
                        break;
                    }
                    case 3: {
                        rightButton.setToolTipText("Next: Imagetype Bar");
                        break;
                    }
                    case 4: {
                        rightButton.setToolTipText("Next: Transform Bar");
                        break;
                    }
                    case 5: {
                        rightButton.setToolTipText("Next: Adjust Bar");
                        break;
                    }
                    case 6: {
                        rightButton.setToolTipText("Next: Movie Bar");
                        break;
                    }
                    case 7: {
                        rightButton.setToolTipText("Next: Command Bar");
                        break;
                    }
                    case 8: {
                        rightButton.setToolTipText("Next: LUT Bar");
                        break;
                    }
                    }
                 }
                 else{
                     //increment--;
                     //if(increment < 1)increment=length;
                 }
                 this.firePropertyChange("incrementId", oldInc, increment);
                 oldInc=increment;*/
                Thread.sleep(2500);
             } catch (Exception e) {
                e.printStackTrace();
            }
        }
        timer = null;
    }


}
