package ij2x.plugin.filter;


import ij.*;
import ij.gui.*;
import ij.process.*;
import java.awt.*;
import java.awt.geom.*;
import ij.plugin.filter.*;
import javax.swing.JLabel;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import java.util.Vector;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

/** This plugin implements the Image/Rotate/Arbitrarily command. */
public class RotateAll implements ExtendedPlugInFilter, DialogListener, ChangeListener {
        private int flags = DOES_ALL|SUPPORTS_MASKING|PARALLELIZE_STACKS;
        private static double angle = 0.0;
        private double curAngle=0.0, oldangle=0.0;
        private static boolean interpolate = true;
        private static boolean fillWithBackground;
        private static boolean enlarge;
        private static int gridLines = 1;
        private ImagePlus imp;
        private int bitDepth;
        boolean canEnlarge;
        boolean isEnlarged, isChanged;
        private JSlider jslider;
        private JSpinner jspinner;
        private JCheckBox previewCb, bgCb, enlargeCb, interpolateCb;
        private Vector<JComponent> slider, spinner, checkBoxes;
        private GenericDialog gd;
        private PlugInFilterRunner pfr;
        private BevelBorder bb ;//new BevelBorder for LOWER or RAISED type
        private Border grayBorder = BorderFactory.createBevelBorder(bb.RAISED, Color.gray, Color.lightGray);
        private Roi roi;
        private Graphics g;
        private double[] matrix = new double[6];
        private double rx =0, ry=0, rw=0, rh=0;
        private double mag;
        protected Rectangle srcRect;
        private ImageCanvas ic;

        /**
         * The <code>AffineTransform</code> class represents a 2D affine transform
         * that performs a linear mapping from 2D coordinates to other 2D
         * coordinates that preserves the "straightness" and
         * "parallelness" of lines.  Affine transformations can be constructed
         * using sequences of translations, scales, flips, rotations, and shears.
         * <p>
         * Such a coordinate transformation can be represented by a 3 row by
         * 3 column matrix with an implied last row of [ 0 0 1 ].  This matrix
         * transforms source coordinates <code>(x,&nbsp;y)</code> into
         * destination coordinates <code>(x',&nbsp;y')</code> by considering
         * them to be a column vector and multiplying the coordinate vector
         * by the matrix according to the following process:
         * <pre>
         *	[ x']   [  m00  m01  m02  ] [ x ]   [ m00x + m01y + m02 ]
         *	[ y'] = [  m10  m11  m12  ] [ y ] = [ m10x + m11y + m12 ]
         *	[ 1 ]   [   0    0    1   ] [ 1 ]   [         1         ]
         * </pre>
         *
         * @version 1.71, 12/19/03
         * @author Jim Graham
         */


        public int setup(String arg, ImagePlus imp) {
                this.imp = imp;
                if (imp!=null) {
                        bitDepth = imp.getBitDepth();
                        g=imp.getCanvas().getGraphics();
                        ic = imp.getCanvas();
                        srcRect = ic.getSrcRect();
                        //System.out.println("srcRect=" + srcRect);
                        roi = imp.getRoi();
                        mag = imp.getCanvas().getMagnification();
                        Rectangle r = roi!=null?roi.getBounds():null;
                        canEnlarge = r==null || (r.x==0&&r.y==0&&r.width==imp.getWidth()&&r.height==imp.getHeight());
                }
                return flags;
        }

        public void run(ImageProcessor ip) {
                if(enlarge && gd.wasOKed()) synchronized(this) {
                        if (!isEnlarged) {
                            if (imp.getStackSize()==1)
                                    Undo.setup(Undo.COMPOUND_FILTER, imp);
                                IJ.run("Select All");
                                Roi roi = imp.getRoi();
                                Rectangle r = roi.getBounds();
                                IJ.showStatus("Enlarging...");
                                double radius = Math.sqrt(r.width*r.width+r.height*r.height);
                                IJ.run("Canvas Size...", "width="+radius+" height="+radius+" position=Center "+(fillWithBackground?"":"zero"));
                            IJ.showStatus("Rotating...");
                                isEnlarged=true;
                        }
                }
                if (isEnlarged) {	//enlarging may have made the ImageProcessor invalid, also for the parallel threads
                        int slice = pfr.getSliceNumber();
                        if (imp.getStackSize()==1)
                                ip = imp.getProcessor();
                        else
                                ip = imp.getStack().getProcessor(slice);
                }
                ip.setInterpolate(interpolate);
                if (fillWithBackground) {
                        Color bgc = Toolbar.getBackgroundColor();
                        if (bitDepth==8)
                                ip.setBackgroundValue(ip.getBestIndex(bgc));
                        else if (bitDepth==24)
                                ip.setBackgroundValue(bgc.getRGB());
                } else
                        ip.setBackgroundValue(0);
                ip.rotate(angle);
                if (!gd.wasOKed())
                        drawGridLines(gridLines);
                if (isEnlarged && imp.getStackSize()==1) {
                        imp.changes = true;
                        imp.updateAndDraw();
                        Undo.setup(Undo.COMPOUND_FILTER_DONE, imp);
                }
        }

        void enlargeCanvas() {
                imp.unlock();
                if (imp.getStackSize()==1)
                        Undo.setup(Undo.COMPOUND_FILTER, imp);
                    IJ.run("Select All");
                    Roi roi = imp.getRoi();
                    Rectangle r = roi.getBounds();
                    IJ.showStatus("Enlarging...");
                    double radius = Math.sqrt(r.width*r.width+r.height*r.height);
                    IJ.run("Canvas Size...", "width="+radius+" height="+radius+" position=Center "+(fillWithBackground?"":"zero"));
                IJ.showStatus("Rotating...");
        }


        void drawGridLines(int lines) {
                if (ic==null) return;
                if (lines==0) {ic.setOverlay(null); return;}
                GeneralPath path = new GeneralPath();
                float width = imp.getWidth();
                float height = imp.getHeight();
                float xinc = width/lines;
                float yinc = height/lines;
                float xstart = xinc/2f;
                float ystart = yinc/2f;
                for (int i=0; i<lines; i++) {
                        path.moveTo(xstart+xinc*i, 0f);
                        path.lineTo(xstart+xinc*i, height);
                        path.moveTo(0f, ystart+yinc*i);
                        path.lineTo(width, ystart+yinc*i);
                }
                //ic.setOverlay(path, Roi.getColor(), null);
                imp.setOverlay(path, Roi.getColor(), null);
        }

        public int showDialog(ImagePlus imp, String command, PlugInFilterRunner pfr) {
                this.pfr = pfr;
                gd = new GenericDialog("Rotate", IJ.getInstance());
                String angleLabel="Angle:";
                int min=0, max=360;
                int value=(int)angle;
                int mior=90/2;
                int major=90;
                int width=164, height=58;
                int stepSize=1;
                boolean paintticks=true, paintLabels=true;
                JPanel transformPanel = new JPanel();
                JLabel transformLabel = new JLabel(" Rotate ");
                transformPanel.add(transformLabel);
                transformLabel.setBorder(grayBorder);
                gd.addPanel(transformPanel, GridBagConstraints.WEST, new Insets(5, 5, 5, 5));
                gd.addJSlider(angleLabel, min, max, value, major, mior, paintticks, paintLabels, width, height, stepSize);
                gd.addNumericField("Grid Lines: ", gridLines, 0);
                gd.addCheckbox("Interpolate", interpolate);
                spinner=gd.getSpinners();
                slider =gd.getSliders();
                jspinner=(JSpinner) spinner.elementAt(0);
                jslider=(JSlider) slider.elementAt(0);
                jslider.addChangeListener(this);
                jspinner.addChangeListener(this);
                if (bitDepth==8 || bitDepth==24)
                        gd.addCheckbox("Fill with Background Color", fillWithBackground);
                if (canEnlarge)
                        gd.addCheckbox("Enlarge Image to Fit Result", enlarge);
                else
                        enlarge = false;
                gd.addPreviewCheckbox(pfr);
                gd.addDialogListener(this);
                checkBoxes=gd.getCheckboxes();
                previewCb=(JCheckBox)checkBoxes.elementAt(0);
                previewCb.addChangeListener(this);
                gd.showDialog();
                drawGridLines(0);
                if (gd.wasCanceled()) {
                    angle=curAngle;
                        return DONE;
                }
                curAngle=angle;
                if (!enlarge)
                        flags |= KEEP_PREVIEW;		// standard filter without enlarge
                else if (imp.getStackSize()==1)
                        flags |= NO_CHANGES;			// undoable as a "compound filter"
                return IJ.setupDialog(imp, flags);
        }

        void rotate2DRoi(double angle) {
            Graphics2D g2D = (Graphics2D)g;
            Polygon p =new Polygon();
            double x, y, ax=0.0, ay=0.0, d2, d, width, height, xcenter, ycenter;
            double arg, a, argt;
            int type=0, npoints=4;
                if (roi!=null) {
                        p = roi.getPolygon();
                        //if(p == null)return;
                        npoints=p.npoints;
                        Rectangle r = roi.getBounds();
                        x=(r.x);
                        y=(r.y);
                        width=(r.width);
                        height=(r.height);
                        type=roi.getType();
                } else {
                    //if(p == null)return;
                        x=(0);
                        y=(0);
                        width=(imp.getWidth());
                        height=(imp.getHeight());
                        type=0;
                }
                AffineTransform at = new AffineTransform();
                xcenter=x+width/2;
                ycenter=y+height/2;
                xcenter*=mag;
                ycenter*=mag;
                //System.out.println("p.npoints="+p.npoints);
                double[] xa = new double[npoints];
                double[] ya = new double[npoints];
                int[] xam = new int[npoints];
                int[] yam = new int[npoints];
                for (int i=0; i<npoints; i++){
                    xa[i] = p.xpoints[i]*mag;
                    ya[i] = p.ypoints[i]*mag;
                    xam[i]=(int)Math.round(xa[i]);
                    yam[i]=(int)Math.round(ya[i]);
                }
                    g2D.translate(-srcRect.x*mag, -srcRect.y*mag);
                    at.setToRotation(angle*(Math.PI/180.0), xcenter, ycenter);
                    g2D.transform(at);
                    g2D.setColor(Roi.getColor());
                    g2D.drawPolygon(xam, yam, npoints);
                    g2D.translate(srcRect.x*mag, srcRect.y*mag);
                imp.draw();
        }

        public boolean dialogItemChanged(GenericDialog gd, AWTEvent e) {
                angle = jslider.getValue();
                //only check for invalid input to "angle", don't care about gridLines
                if (gd.invalidNumber()) {
                        if (gd.wasOKed()) IJ.error("Angle is invalid.");
                        return false;
                }
                gridLines = (int)gd.getNextNumber();
                interpolate = previewCb.isSelected();
                    gd.getNextBoolean();
                if (bitDepth==8 || bitDepth==24)
                        fillWithBackground = gd.getNextBoolean();
                if (canEnlarge)
                        enlarge = gd.getNextBoolean();
                 return true;
        }

        public void setNPasses(int nPasses) {
        }
        public synchronized void stateChanged(ChangeEvent e) {
            previewCb.setSelected(!interpolate);
            if(e.getSource() instanceof JSlider)
             {angle = jslider.getValue();
            //System.out.println("jslider_angle="+angle);
        }
            else if(e.getSource() instanceof JSpinner)
             {angle = ((Integer)jspinner.getValue()).intValue();
             //System.out.println("jspinner_angle="+angle);
         }
             double difangle = angle-oldangle;
             //isChanged=!isChanged;
             if(difangle != 0)
             rotate2DRoi(difangle);
             oldangle=angle;
        }

}

